@@ -45,20 +45,22 @@ Tile caching (creation, caching and retrieval of tiles).
 
 from __future__ import with_statement
 import os
+import sys
 import time
 import errno
 import hashlib
 from functools import partial
 
 from mapproxy.core.utils import FileLock, cleanup_lockdir, ThreadedExecutor
-from mapproxy.core.image import TiledImage, ImageSource
+from mapproxy.core.image import TiledImage, ImageSource, is_single_color_image
 from mapproxy.core.config import base_config, abspath
+from mapproxy.core.grid import NoTiles
 
 import logging
 log = logging.getLogger(__name__)
 
-
-
+class BlankImage(Exception):
+    pass
 class TileCacheError(Exception):
     pass
 class TileSourceError(TileCacheError):
@@ -111,8 +113,13 @@ class Cache(object):
         :param out_size: the target output size
         :rtype: `ImageSource`
         """
-        src_bbox, tile_grid, affected_tile_coords = \
-            self.grid.get_affected_tiles(req_bbox, out_size, req_srs=req_srs)
+        try:
+            src_bbox, tile_grid, affected_tile_coords = \
+                self.grid.get_affected_tiles(req_bbox, out_size, req_srs=req_srs)
+        except IndexError:
+            raise TileCacheError('Invalid BBOX')
+        except NoTiles:
+            raise BlankImage()
         
         num_tiles = tile_grid[0] * tile_grid[1]
         if num_tiles >= base_config().cache.max_tile_limit:
@@ -170,11 +177,14 @@ class CacheManager(object):
         self.cache = cache
         self.tile_source = tile_source
         self.tile_creator = tile_creator
+        self._expire_timestamp = None
         
     def is_cached(self, tile):
         """
         Return True if the tile is cached.
         """
+        if isinstance(tile, tuple):
+            tile = _Tile(tile)
         max_mtime = self.expire_timestamp(tile)
         cached = self.cache.is_cached(tile)
         if cached and max_mtime is not None:
@@ -188,10 +198,9 @@ class CacheManager(object):
         Return the timestamp until which a tile should be accepted as up-to-date,
         or ``None`` if the tiles should not expire.
         
-        :note: Returns ``None`` by default. Overwrite/change method to enable
-            expiration.
+        :note: Returns _expire_timestamp by default.
         """
-        return None
+        return self._expire_timestamp
     
     def load_tile_coords(self, tile_coords, with_metadata=False):
         """
@@ -253,7 +262,8 @@ class FileCache(object):
     """
     This class is responsible to store and load the actual tile data.
     """
-    def __init__(self, cache_dir, file_ext, pre_store_filter=None):
+    def __init__(self, cache_dir, file_ext, pre_store_filter=None,
+                 link_single_color_images=False):
         """
         :param cache_dir: the path where the tile will be stored
         :param file_ext: the file extension that will be appended to
@@ -267,6 +277,10 @@ class FileCache(object):
         if pre_store_filter is None:
             pre_store_filter = []
         self.pre_store_filter = pre_store_filter
+        if link_single_color_images and sys.platform == 'win32':
+            log.warn('link_single_color_images not supported on windows')
+            link_single_color_images = False
+        self.link_single_color_images = link_single_color_images
     
     def level_location(self, level):
         """
@@ -305,6 +319,22 @@ class FileCache(object):
             _create_dir(tile.location)
         return tile.location
     
+    def _single_color_tile_location(self, color, create_dir=False):
+        """
+        >>> c = FileCache(cache_dir='/tmp/cache/', file_ext='png')
+        >>> c._single_color_tile_location((254, 0, 4)).replace('\\\\', '/')
+        '/tmp/cache/single_color_tiles/fe0004.png'
+        """
+        parts = (
+            self.cache_dir,
+            'single_color_tiles',
+            ''.join('%02x' % v for v in color) + '.' + self.file_ext
+        )
+        location = os.path.join(*parts)
+        if create_dir:
+            _create_dir(location)
+        return location
+    
     def timestamp_created(self, tile):
         """
         Return the timestamp of the last modification of the tile.
@@ -314,7 +344,7 @@ class FileCache(object):
     
     def _update_tile_metadata(self, tile):
         location = self.tile_location(tile)
-        stats = os.stat(tile.location)
+        stats = os.lstat(location)
         tile.timestamp = stats.st_mtime
         tile.size = stats.st_size
     
@@ -358,12 +388,39 @@ class FileCache(object):
         """
         if tile.stored:
             return
+        
         tile_loc = self.tile_location(tile, create_dir=True)
+        
+        if self.link_single_color_images:
+            color = is_single_color_image(tile.source.as_image())
+            if color:
+                real_tile_loc = self._single_color_tile_location(color, create_dir=True)
+                if not os.path.exists(real_tile_loc):
+                    self._store(tile, real_tile_loc)
+                
+                log.debug('linking %r from %s to %s',
+                          tile.coord, real_tile_loc, tile_loc)
+                
+                # remove any file before symlinking.
+                # exists() returns False if it links to non-
+                # existing file, islink() test to check that
+                if os.path.exists(tile_loc) or os.path.islink(tile_loc):
+                    os.unlink(tile_loc)
+                
+                os.symlink(real_tile_loc, tile_loc)
+                return
+        
+        self._store(tile, tile_loc)
+    
+    def _store(self, tile, location):
+        if os.path.islink(location):
+            os.unlink(location)
+        
         for img_filter in self.pre_store_filter:
             tile = img_filter(tile)
         data = tile.source.as_buffer()
-        with open(tile_loc, 'wb') as f:
-            log.debug('writing %r to %s' % (tile.coord, tile_loc))
+        with open(location, 'wb') as f:
+            log.debug('writing %r to %s' % (tile.coord, location))
             f.write(data.read())
         tile.size = data.tell()
         tile.timestamp = time.time()
@@ -437,6 +494,7 @@ class _ThreadedTileCreator(_TileCreator):
             new_tiles = self._create_tile(unique_meta_tiles[0], tile_collection)
             if new_tiles is None:
                 return []
+            cleanup_lockdir(self.tile_source.lock_dir)
             return new_tiles
         else:
             return self._create_multiple_tiles(unique_meta_tiles, tile_collection)
@@ -504,7 +562,8 @@ class TileSource(object):
         Returns a lock object for the given tile.
         """
         lock_file = self.lock_filename(tile)
-        return FileLock(lock_file)
+        # TODO use own configuration option for lock timeout
+        return FileLock(lock_file, timeout=base_config().http_client_timeout)
     
     def lock_filename(self, tile):
         if self._id is None:
