@@ -19,14 +19,19 @@ server = ['wms', 'tms', 'kml']
 wms = dict(
     image_formats = ['image/jpeg', 'image/png', 'image/gif', 'image/GeoTIFF', 'image/tiff'],
     srs = set(['EPSG:4326', 'EPSG:4258', 'CRS:84', 'EPSG:900913', 'EPSG:31466', 'EPSG:31467', 'EPSG:31468', 'EPSG:25831', 'EPSG:25832', 'EPSG:25833']),
+    non_strict = False,
     request_parser = 'default',
     client_request = 'default',
 )
 debug_mode = False
 
 srs = dict(
-    axis_order_ne = set(['EPSG:4326', 'EPSG:4258', 'EPSG:31466', 'EPSG:31467', 'EPSG:31468']),
-    axis_order_en = set(['CRS:84', 'EPSG:900913', 'EPSG:25831', 'EPSG:25832', 'EPSG:25833']),
+    # user sets
+    axis_order_ne = set(),
+    axis_order_en = set(),
+    # default sets, both will be combined in config:load_base_config
+    axis_order_ne_ = set(['EPSG:4326', 'EPSG:4258', 'EPSG:31466', 'EPSG:31467', 'EPSG:31468']),
+    axis_order_en_ = set(['CRS:84', 'EPSG:900913', 'EPSG:25831', 'EPSG:25832', 'EPSG:25833']),
 )
 
 image = dict(
@@ -34,6 +39,7 @@ image = dict(
     resampling_method = 'bicubic',
     jpeg_quality = 90,
     stretch_factor = 1.15,
+    max_shrink_factor = 4.0,
 )
 # number of concurrent requests to a tile source
 tile_creator_pool_size = 2
@@ -52,4 +58,11 @@ tiles = dict(
     expires_hours = 72,
 )
 
+http = dict(
+    ssl = dict(
+        ca_certs = None,
+        insecure = False,
+    ),
+)
+
 http_client_timeout = 60
