@@ -26,7 +26,10 @@ log = logging.getLogger(__name__)
 from mapproxy.core.config import base_config
 from mapproxy.core.srs import SRS, get_epsg_num
 
-geodetic_epsg_codes = [4326, 31466, 31467, 31468, 31469]
+geodetic_epsg_codes = [4326]
+
+class NoTiles(Exception):
+    pass
 
 def get_resolution(bbox, size):
     """
@@ -91,7 +94,10 @@ class TileGrid(object):
         self.is_geodetic = is_geodetic
         
         self.stretch_factor = base_config().image.stretch_factor
-        'allow images to be scaled by this factor before the next level will be selected'
+        'allow images to be scaled up by this factor before the next level will be selected'
+        
+        self.max_shrink_factor = base_config().image.max_shrink_factor
+        'allow images to be scaled down by this factor before NoTiles is raised'
         
         if levels is None:
             self.levels = 20
@@ -233,25 +239,50 @@ class TileGrid(object):
         ... #doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
         ((-20037508.342789244, -20037508.342789244,\
           20037508.342789244, 20037508.342789244), (1, 1),\
-          <generator object at ...>)
+          <generator object ...>)
         """
+        src_bbox, level = self.get_affected_bbox_and_level(bbox, size, req_srs=req_srs)
+        return self.get_affected_level_tiles(src_bbox, level, inverse=inverse)
+    
+    def get_affected_bbox_and_level(self, bbox, size, req_srs=None):
         if req_srs and req_srs != self.srs:
             src_bbox = req_srs.transform_bbox_to(self.srs, bbox)
         else:
             src_bbox = bbox
         
+        if not bbox_intersects(self.bbox, src_bbox):
+            raise NoTiles()
+        
         res = get_resolution(src_bbox, size)
         level = self.closest_level(res)
+        
+        if res > self.resolutions[level]*self.max_shrink_factor:
+            raise NoTiles()
+        
+        return src_bbox, level
+    
+    def get_affected_level_tiles(self, bbox, level, inverse=False):
+        """
+        Get a list with all affected tiles for a `bbox` in the given `level`.
+        :returns: the bbox, the size and a list with tile coordinates, sorted row-wise
+        :rtype: ``bbox, (xs, yz), [(x, y, z), ...]``
+
+        >>> grid = TileGrid()
+        >>> bbox = (-20037508.34, -20037508.34, 20037508.34, 20037508.34)
+        >>> grid.get_affected_level_tiles(bbox, 0)
+        ... #doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
+        ((-20037508.342789244, -20037508.342789244,\
+          20037508.342789244, 20037508.342789244), (1, 1),\
+          <generator object ...>)
+        """
         # remove 1/10 of a pixel so we don't get a tiles we only touch
-        x_delta = (src_bbox[2]-src_bbox[0]) / size[0] / 10.0
-        y_delta = (src_bbox[3]-src_bbox[1]) / size[1] / 10.0
-        x0, y0, _ = self.tile(src_bbox[0]+x_delta, src_bbox[1]+y_delta, level)
-        x1, y1, _ = self.tile(src_bbox[2]-x_delta, src_bbox[3]-y_delta, level)
+        delta = self.resolutions[level] / 10.0
+        x0, y0, _ = self.tile(bbox[0]+delta, bbox[1]+delta, level)
+        x1, y1, _ = self.tile(bbox[2]-delta, bbox[3]-delta, level)
         
-        log.debug('BBOX: ' + str(src_bbox))
-        log.debug('coords: %f, %f, %f, %f' % (x0, y0, x1, y1))
-        log.debug('res: ' + str(res))
+        return self._tile_iter(x0, y0, x1, y1, level, inverse=inverse)
         
+    def _tile_iter(self, x0, y0, x1, y1, level, inverse=False):
         xs = range(x0, x1+1)
         if inverse:
             y0 = int(self.grid_sizes[level][1]) - 1 - y0
@@ -267,7 +298,7 @@ class TileGrid(object):
         abbox = self._get_bbox([ll, ur])
         return (abbox, (len(xs), len(ys)),
                 _create_tile_list(xs, ys, level, self.grid_sizes[level]))
-    
+        
     def _get_bbox(self, tiles):
         """
         Returns the bbox of multiple tiles.
@@ -490,4 +521,82 @@ class MetaGrid(object):
     def meta_size(self, level):
         grid_size = self.grid.grid_sizes[level]
         return min(self._meta_size[0], grid_size[0]), min(self._meta_size[1], grid_size[1])
+    
+    
+    def get_affected_level_tiles(self, bbox, level):
+        """
+        Get a list with all affected tiles for a `bbox` in the given `level`.
+        
+        :returns: the bbox, the size and a list with tile coordinates, sorted row-wise
+        :rtype: ``bbox, (xs, yz), [(x, y, z), ...]``
+        
+        >>> grid = MetaGrid(TileGrid(), (2, 2))
+        >>> bbox = (-20037508.34, -20037508.34, 20037508.34, 20037508.34)
+        >>> grid.get_affected_level_tiles(bbox, 0)
+        ... #doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
+        ((-20037508.342789244, -20037508.342789244,\
+          20037508.342789244, 20037508.342789244), (1, 1),\
+          <generator object ...>)
+        """
+        
+        # remove 1/10 of a pixel so we don't get a tiles we only touch
+        delta = self.grid.resolutions[level] / 10.0
+        x0, y0, _ = self.grid.tile(bbox[0]+delta, bbox[1]+delta, level)
+        x1, y1, _ = self.grid.tile(bbox[2]-delta, bbox[3]-delta, level)
+        
+        meta_size = self.meta_size(level)
+        
+        x0 = x0//meta_size[0] * meta_size[0]
+        x1 = x1//meta_size[0] * meta_size[0]
+        y0 = y0//meta_size[1] * meta_size[1]
+        y1 = y1//meta_size[1] * meta_size[1]
+        
+        return self._tile_iter(x0, y0, x1, y1, level)
+        
+    def _tile_iter(self, x0, y0, x1, y1, level, inverse=False):
+        meta_size = self.meta_size(level)
+        
+        xs = range(x0, x1+1, meta_size[0])
+        if inverse:
+            y0 = int(self.grid.grid_sizes[level][1]) - 1 - y0
+            y1 = int(self.grid.grid_sizes[level][1]) - 1 - y1
+            ys = range(y1, y0+1, meta_size[1])
+        else:
+            ys = range(y1, y0-1, -1 * meta_size[1])
+        ll = (xs[0], ys[-1], level)
+        ur = (xs[-1], ys[0], level)
+        if inverse:
+            ll = self.grid.flip_tile_coord(ll)
+            ur = self.grid.flip_tile_coord(ur)
+        # add meta_size to get full affected bbox
+        ur = ur[0]+meta_size[0]-1, ur[1]+meta_size[1]-1, ur[2]
+        abbox = self.grid._get_bbox([ll, ur])
+        return (abbox, (len(xs), len(ys)),
+                _create_tile_list(xs, ys, level, self.grid.grid_sizes[level]))
 
+
+def bbox_intersects(one, two):
+    a_x0, a_y0, a_x1, a_y1 = one
+    b_x0, b_y0, b_x1, b_y1 = two
+    
+    if (
+        a_x0 < b_x1 and
+        a_x1 > b_x0 and
+        a_y0 < b_y1 and
+        a_y1 > b_y0
+        ): return True
+    
+    return False
+
+def bbox_contains(one, two):
+    a_x0, a_y0, a_x1, a_y1 = one
+    b_x0, b_y0, b_x1, b_y1 = two
+    
+    if (
+        a_x0 < b_x0 and
+        a_x1 > b_x1 and
+        a_y0 < b_y0 and
+        a_y1 > b_y1
+        ): return True
+    
+    return False
