@@ -49,13 +49,13 @@ class LayerMerger(object):
         Add one or more layers to merge. Bottom-layers first.
         """
         try:
-            layers = iter(layer)
+            layer = iter(layer)
         except TypeError:
             if layer is not None:
                 self.layers.append(layer)
         else:
-            [self.layers.append(layer) for layer in layers if layer is not None]
-        
+            for l in layer:
+                self.add(l)
 
     def merge(self, format='png', size=None, bgcolor='#ffffff', transparent=False):
         """
@@ -132,6 +132,11 @@ class ImageSource(object):
             f = self.source
             if isinstance(f, basestring):
                 f = open(f, 'rb')
+            elif not hasattr(f, 'seek'):
+                # PIL needs file objects with seek
+                f = StringIO(f.read())
+                self.source = f
+                f.seek(0)
             try:
                 return Image.open(f)
             except StandardError:
@@ -159,6 +164,7 @@ class ImageSource(object):
             return open(self.source, 'rb')
         if not hasattr(self.source, 'seek'):
             return ReadBufWrapper(self.source)
+        self.source.seek(0)
         return self.source
     @property
     def size(self):
@@ -476,6 +482,7 @@ class TileMerger(object):
         """
         self.tile_grid = tile_grid
         self.tile_size = tile_size
+    
     def merge(self, ordered_tiles, transparent=False):
         """
         Merge all tiles into one image.
@@ -490,14 +497,22 @@ class TileMerger(object):
                 return ImageSource(tile.source, size=self.tile_size,
                                    transparent=transparent)
         src_size = self._src_size()
-        result = Image.new("RGBA", src_size, (255, 255, 255, 255))
+        
+        if transparent:
+            result = Image.new("RGBA", src_size, (255, 255, 255, 0))
+        else:
+            result = Image.new("RGB", src_size, (255, 255, 255))
+
         for i, source in enumerate(ordered_tiles):
             if source is None:
                 continue
             try:
                 tile = source.as_image()
-                tile.draft('RGBA', self.tile_size)
                 pos = self._tile_offset(i)
+                if transparent:
+                    tile.draft('RGBA', self.tile_size)
+                else:
+                    tile.draft('RGB', self.tile_size)
                 result.paste(tile, pos)
             except IOError, e:
                 log.warn('unable to load tile %s, removing it (reason was: %s)'
@@ -506,10 +521,12 @@ class TileMerger(object):
                     if os.path.exists(source.source):
                         os.remove(source.source)
         return ImageSource(result, size=src_size, transparent=transparent)
+    
     def _src_size(self):
         width = self.tile_grid[0]*self.tile_size[0]
         height = self.tile_grid[1]*self.tile_size[1]
         return width, height
+    
     def _tile_offset(self, i):
         """
         Return the image offset (upper-left coord) of the i-th tile,
@@ -625,11 +642,18 @@ class ImageTransformer(object):
         minx, miny = to_src_px((dst_bbox[0], dst_bbox[3]))
         maxx, maxy = to_src_px((dst_bbox[2], dst_bbox[1]))
         
-        src_res = (src_bbox[0]-src_bbox[2])/src_img.size[0]
-        dst_res = (dst_bbox[0]-dst_bbox[2])/dst_size[0]
+        src_res = ((src_bbox[0]-src_bbox[2])/src_img.size[0],
+                   (src_bbox[1]-src_bbox[3])/src_img.size[1])
+        dst_res = ((dst_bbox[0]-dst_bbox[2])/dst_size[0],
+                   (dst_bbox[1]-dst_bbox[3])/dst_size[1])
         
-        tenth_px_res = abs(dst_res/(dst_size[0]*10))
-        if abs(src_res-dst_res) < tenth_px_res:
+        tenth_px_res = (abs(dst_res[0]/(dst_size[0]*10)),
+                        abs(dst_res[1]/(dst_size[1]*10)))
+        if (abs(src_res[0]-dst_res[0]) < tenth_px_res[0] and
+            abs(src_res[1]-dst_res[1]) < tenth_px_res[1]):
+            # rounding might result in subpixel inaccuracy
+            # this exact resolutioni match should only happen in clients with
+            # fixed resolutions like OpenLayers
             minx = int(round(minx))
             miny = int(round(miny))
             result = src_img.as_image().crop((minx, miny,
@@ -656,7 +680,7 @@ class ImageTransformer(object):
             src_quad = []
             for dst_px in [(quad[0], quad[1]), (quad[0], quad[3]),
                            (quad[2], quad[3]), (quad[2], quad[1])]:
-                dst_w = to_dst_w(dst_px)
+                dst_w = to_dst_w((dst_px[0]+0.5, dst_px[1]+0.5))
                 src_w = self.dst_srs.transform_to(self.src_srs, dst_w)
                 src_px = to_src_px(src_w)
                 src_quad.extend(src_px)
@@ -669,7 +693,7 @@ class ImageTransformer(object):
             meshes.append(dst_quad_to_src(quad))
         result = src_img.as_image().transform(dst_size, Image.MESH, meshes,
                                               image_filter[self.resampling])
-        return ImageSource(result, size=self.dst_size, transparent=src_img.transparent)
+        return ImageSource(result, size=dst_size, transparent=src_img.transparent)
     
     def _no_transformation_needed(self, src_size, src_bbox, dst_size, dst_bbox):
         """
@@ -752,3 +776,20 @@ class TiledImage(object):
         src_img = self.image()
         return transformer.transform(src_img, self.src_bbox, out_size, req_bbox)
     
+def is_single_color_image(image):
+    """
+    Checks if the `image` contains only one color.
+    Returns ``False`` if it contains more than one color, else
+    the color-tuple of the single color.
+    """
+    result = image.getcolors(1)
+    # returns a list of (count, color), limit to one
+    if result is None:
+        return False
+    
+    color = result[0][1]
+    if image.mode == 'P':
+        palette = image.getpalette()
+        return palette[color*3], palette[color*3+1], palette[color*3+2]
+    
+    return result[0][1]
\ No newline at end of file
