@@ -26,7 +26,6 @@ import Queue
 import threading
 import shutil
 import datetime
-from subprocess import Popen, PIPE
 from mapproxy.core import lockfile
 
 import logging
@@ -86,13 +85,17 @@ def cleanup_lockdir(lockdir, suffix='.lck', max_lock_time=300):
         return
     for entry in os.listdir(lockdir):
         name = os.path.join(lockdir, entry)
-        if os.path.isfile(name) and name.endswith(suffix):
-            if os.path.getmtime(name) < expire_time:
-                try:
-                    os.unlink(name)
-                except IOError, ex:
-                    log.warn('could not remove old lock file %s: %s', name, ex)
-                
+        try:
+            if os.path.isfile(name) and name.endswith(suffix):
+                if os.path.getmtime(name) < expire_time:
+                    try:
+                        os.unlink(name)
+                    except IOError, ex:
+                        log.warn('could not remove old lock file %s: %s', name, ex)
+        except OSError, e:
+            # some one might remove the file, ignore this
+            if e.errno != errno.ENOENT:
+                raise e
 
 class ThreadedExecutor(object):
     class Executor(threading.Thread):
@@ -264,10 +267,10 @@ def timestamp_from_isodate(isodate):
     """
     >>> timestamp_from_isodate('2009-06-09T10:57:00')
     1244537820.0
-    >>> timestamp_from_isodate('2009-06-09T10:57')
+    >>> timestamp_from_isodate('2009-06-09T10:57') #doctest: +ELLIPSIS
     Traceback (most recent call last):
-      ...
-    ValueError: time data did not match format:  data=2009-06-09T10:57  fmt=%Y-%m-%dT%H:%M:%S
+    ...
+    ValueError: ...
     """
     date = datetime.datetime.strptime(isodate, "%Y-%m-%dT%H:%M:%S")
     return time.mktime(date.timetuple())
@@ -285,7 +288,7 @@ def cleanup_directory(directory, before_timestamp, remove_empty_dirs=True,
                 continue
             for filename in filenames:
                 filename = os.path.join(dirpath, filename)
-                if os.stat(filename).st_mtime < before_timestamp:
+                if os.lstat(filename).st_mtime < before_timestamp:
                     file_handler(filename)
             if (remove_empty_dirs and not os.listdir(dirpath)
                 and dirpath != directory):
