@@ -16,6 +16,7 @@
 
 from __future__ import with_statement
 import os
+import sys
 import Image
 import functools
 
@@ -26,13 +27,12 @@ from mapproxy.core.app import make_wsgi_app
 from mapproxy.wms.request import WMS100MapRequest, WMS111MapRequest, WMS130MapRequest, \
                                  WMS111FeatureInfoRequest, WMS111CapabilitiesRequest, \
                                  WMS130CapabilitiesRequest, WMS100CapabilitiesRequest, \
-                                 WMS100FeatureInfoRequest
+                                 WMS100FeatureInfoRequest, WMS130FeatureInfoRequest
 from mapproxy.tests.image import is_jpeg, is_png, tmp_image
 from mapproxy.tests.http import mock_httpd
 from mapproxy.tests.helper import validate_with_dtd, validate_with_xsd
 from nose.tools import eq_
 
-
 global_app = None
 
 def setup_module():
@@ -47,7 +47,7 @@ def setup_module():
     mapproxy.core.config._service_config = None
     
     global global_app
-    global_app = TestApp(make_wsgi_app(init_logging=False))
+    global_app = TestApp(make_wsgi_app())
 
 def teardown_module():
     mapproxy.core.config._config = None
@@ -66,12 +66,13 @@ class WMSTest(object):
         for filename in self.created_tiles:
             yield os.path.join(base_dir, filename)
     
-    def test_created_tiles(self):
+    def _test_created_tiles(self):
         for filename in self.created_tiles_filenames():
             if not os.path.exists(filename):
                 assert False, "didn't found tile " + filename
     
     def teardown(self):
+        self._test_created_tiles()
         for filename in self.created_tiles_filenames():
             if os.path.exists(filename):
                 os.remove(filename)
@@ -142,7 +143,8 @@ class TestWMS111(WMSTest):
             'http://localhost:80/service?')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi'])
+                              'wms_merge', 'tms_cache', 'wms_cache_multi',
+                              'wms_cache_link_single'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.1/WMS_MS_Capabilities.dtd')
     
@@ -242,7 +244,7 @@ class TestWMS111(WMSTest):
         # broken bbox for the requested srs
         url =  """/service?SERVICE=WMS&VERSION=1.1.1&REQUEST=GetMap&BBOX=-72988843.697212,-255661507.634227,142741550.188860,255661507.634227&SRS=EPSG:25833&WIDTH=164&HEIGHT=388&LAYERS=wms_cache&STYLES=&FORMAT=image/png&TRANSPARENT=TRUE"""
         resp = self.app.get(url)
-        is_111_exception(resp.lxml, 'Request too large or invalid BBOX.')
+        is_111_exception(resp.lxml, 'Invalid BBOX')
     
     def test_get_map_broken_bbox(self):
         url = """/service?VERSION=1.1.11&REQUEST=GetMap&SRS=EPSG:31467&BBOX=-10000855.0573254,2847125.18913603,-9329367.42767611,4239924.78564583&WIDTH=130&HEIGHT=62&LAYERS=wms_cache&STYLES=&FORMAT=image/png&TRANSPARENT=TRUE"""
@@ -302,6 +304,58 @@ class TestWMS111(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
+
+    def test_get_featureinfo_info_format(self):
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                  '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
+                                  '&VERSION=1.1.1&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20'
+                                  '&info_format=text%2Fhtml'},
+                        {'body': 'info', 'headers': {'content-type': 'text/html'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_fi_req.params['info_format'] = 'text/html'
+            resp = self.app.get(self.common_fi_req)
+            eq_(resp.content_type, 'text/html')
+            eq_(resp.body, 'info')
+    
+    def test_get_featureinfo_130(self):
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                  '&REQUEST=GetFeatureInfo&HEIGHT=200&CRS=EPSG%3A900913'
+                                  '&VERSION=1.3.0&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&I=10&J=20'},
+                        {'body': 'info', 'headers': {'content-type': 'text/plain'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_fi_req.params['layers'] = 'wms_cache_130'
+            self.common_fi_req.params['query_layers'] = 'wms_cache_130'
+            resp = self.app.get(self.common_fi_req)
+            eq_(resp.content_type, 'text/plain')
+            eq_(resp.body, 'info')
+        
+    def test_get_featureinfo_missing_params(self):
+        del self.common_fi_req.params['format']
+        del self.common_fi_req.params['styles']
+        resp = self.app.get(self.common_fi_req)
+        xml = resp.lxml
+        assert 'missing parameters' in xml.xpath('//ServiceException/text()')[0]
+        assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
+    
+    def test_get_featureinfo_missing_params_non_strict(self):
+        mapproxy.core.config.base_config().wms.non_strict = True
+        try:
+            expected_req = (
+                {'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                          '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
+                          '&VERSION=1.1.1&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                          '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20'},
+                {'body': 'info', 'headers': {'content-type': 'text/plain'}})
+            with mock_httpd(('localhost', 42423), [expected_req]):
+                del self.common_fi_req.params['format']
+                del self.common_fi_req.params['styles']
+                resp = self.app.get(self.common_fi_req)
+                eq_(resp.content_type, 'text/plain')
+                eq_(resp.body, 'info')
+        finally:
+            mapproxy.core.config.base_config().wms.non_strict = False
     
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
@@ -336,7 +390,8 @@ class TestWMS100(WMSTest):
             'MapProxy test fixture')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi'])
+                              'wms_merge', 'tms_cache', 'wms_cache_multi',
+                              'wms_cache_link_single'])
         eq_(layer_names, expected_names)
         #TODO srs
         assert validate_with_dtd(xml, dtd_name='wms/1.0.0/capabilities_1_0_0.dtd')
@@ -457,6 +512,10 @@ class TestWMS130(WMSTest):
              version='1.3.0', bbox='0,-180,80,0', width='200', height='200',
              layers='wms_cache', crs='EPSG:4326', format='image/png',
              styles='', request='GetMap'))
+        self.common_fi_req = WMS130FeatureInfoRequest(url='/service?',
+            param=dict(i='10', j='20', width='200', height='200', layers='wms_cache_130',
+                       format='image/png', query_layers='wms_cache_130', styles='',
+                       bbox='1000,400,2000,1400', crs='EPSG:900913'))
         
     def test_wms_capabilities(self):
         req = WMS130CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
@@ -468,7 +527,8 @@ class TestWMS130(WMSTest):
         layer_names = set(xml.xpath('//wms:Layer/wms:Layer/wms:Name/text()',
                                     namespaces=ns130))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-                              'wms_merge', 'tms_cache', 'wms_cache_multi'])
+                              'wms_merge', 'tms_cache', 'wms_cache_multi',
+                              'wms_cache_link_single'])
         eq_(layer_names, expected_names)
         assert is_130_capa(xml)
     
@@ -562,4 +622,64 @@ class TestWMS130(WMSTest):
             with mock_httpd(('localhost', 42423), [expected_req]):
                 self.common_map_req.params['bbox'] = '0,0,180,90' #internal axis-order
                 resp = self.app.get(self.common_map_req)
-                eq_(resp.content_type, 'image/png')
\ No newline at end of file
+                eq_(resp.content_type, 'image/png')
+    
+    def test_get_featureinfo(self):
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                  '&REQUEST=GetFeatureInfo&HEIGHT=200&CRS=EPSG%3A900913'
+                                  '&VERSION=1.3.0&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&I=10&J=20'},
+                        {'body': 'info', 'headers': {'content-type': 'text/plain'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            resp = self.app.get(self.common_fi_req)
+            eq_(resp.content_type, 'text/plain')
+            eq_(resp.body, 'info')
+
+    def test_get_featureinfo_111(self):
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                  '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
+                                  '&VERSION=1.1.1&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20'},
+                        {'body': 'info', 'headers': {'content-type': 'text/plain'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_fi_req.params['layers'] = 'wms_cache'
+            self.common_fi_req.params['query_layers'] = 'wms_cache'
+            resp = self.app.get(self.common_fi_req)
+            eq_(resp.content_type, 'text/plain')
+            eq_(resp.body, 'info')
+
+if sys.platform != 'win32':
+    class TestWMSLinkSingleColorImages(WMSTest):
+        def setup(self):
+            WMSTest.setup(self)
+            self.common_map_req = WMS111MapRequest(url='/service?', param=dict(service='WMS', 
+                 version='1.1.1', bbox='-180,0,0,80', width='200', height='200',
+                 layers='wms_cache_link_single', srs='EPSG:4326', format='image/jpeg',
+                 styles='', request='GetMap'))
+    
+        def test_get_map(self):
+            link_name = 'wms_cache_link_single_EPSG900913/01/000/000/001/000/000/001.png'
+            real_name = 'wms_cache_link_single_EPSG900913/single_color_tiles/fe0005.png'
+            self.created_tiles.append(link_name)
+            self.created_tiles.append(real_name)
+            with tmp_image((256, 256), format='png', color='#fe0005') as img:
+                expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
+                                          '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
+                                          '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
+                                          '&WIDTH=256'},
+                                {'body': img.read(), 'headers': {'content-type': 'image/png'}})
+                with mock_httpd(('localhost', 42423), [expected_req]):
+                    self.common_map_req.params['bbox'] = '0,0,180,90'
+                    resp = self.app.get(self.common_map_req)
+                    eq_(resp.content_type, 'image/jpeg')
+            
+                base_dir = mapproxy.core.config.base_config().cache.base_dir
+                single_loc = os.path.join(base_dir, real_name)
+                tile_loc = os.path.join(base_dir, link_name)
+                assert os.path.exists(single_loc)
+                assert os.path.islink(tile_loc)
+            
+                self.common_map_req.params['format'] = 'image/png'
+                resp = self.app.get(self.common_map_req)
+                eq_(resp.content_type, 'image/png')
+            
\ No newline at end of file
