@@ -17,6 +17,7 @@
 from __future__ import with_statement
 import os
 import time
+import Image
 from contextlib import contextmanager
 from tempfile import mkdtemp
 from shutil import rmtree
@@ -38,6 +39,7 @@ from mapproxy.wms.request import WMS111MapRequest
 from mapproxy.wms.cache import WMSTileSource
 from mapproxy.tms.cache import TMSTileSource
 from mapproxy.core.srs import SRS
+from mapproxy.core.client import TMSClient
 from mapproxy.core.image import ImageSource, TiledImage
 from mapproxy.core.utils import LockTimeout
 from mapproxy.core.grid import tile_grid_for_epsg, TileGrid
@@ -45,7 +47,7 @@ from mapproxy.tests.helper import Mocker, TempFiles
 from mapproxy.tests.http import mock_httpd
 from mapproxy.tests.image import tmp_image, is_png, assert_image_mode
 from mocker import ANY
-from nose.tools import eq_, raises
+from nose.tools import eq_, assert_not_equal, raises
 
 
 class TestCache(Mocker):
@@ -216,6 +218,39 @@ class TestFileCache(object):
         loc = self.cache.tile_location(tile)
         assert not os.path.exists(loc)
     
+    def test_single_color_tile_store(self):
+        img = Image.new('RGB', (256, 256), color='#ff0105')
+        tile = _Tile((0, 0, 0), ImageSource(img))
+        self.cache.link_single_color_images = True
+        self.cache.store(tile)
+        assert self.cache.is_cached(tile)
+        loc = self.cache.tile_location(tile)
+        assert os.path.islink(loc)
+        assert os.path.realpath(loc).endswith('ff0105.png')
+        assert is_png(open(loc, 'rb'))
+        
+        tile2 = _Tile((0, 0, 1), ImageSource(img))
+        self.cache.store(tile2)
+        assert self.cache.is_cached(tile2)
+        loc2 = self.cache.tile_location(tile2)
+        assert os.path.islink(loc2)
+        assert os.path.realpath(loc2).endswith('ff0105.png')
+        assert is_png(open(loc2, 'rb'))
+        
+        assert_not_equal(loc, loc2)
+        assert os.path.samefile(loc, loc2)
+    
+    def test_single_color_tile_store_w_alpha(self):
+        img = Image.new('RGBA', (256, 256), color='#ff0105')
+        tile = _Tile((0, 0, 0), ImageSource(img))
+        self.cache.link_single_color_images = True
+        self.cache.store(tile)
+        assert self.cache.is_cached(tile)
+        loc = self.cache.tile_location(tile)
+        assert os.path.islink(loc)
+        assert os.path.realpath(loc).endswith('ff0105ff.png')
+        assert is_png(open(loc, 'rb'))
+    
     def _create_cached_tile(self, tile):
         loc = self.cache.tile_location(tile, create_dir=True)
         with open(loc, 'w') as f:
@@ -346,7 +381,8 @@ TEST_SERVER_ADDRESS = ('127.0.0.1', 56413)
 
 class TestTMSTileSource(object):
     def setup(self):
-        self.tms = TMSTileSource(TileGrid(), 'http://%s:%s/tms' % TEST_SERVER_ADDRESS)
+        client = TMSClient('http://%s:%s/tms' % TEST_SERVER_ADDRESS)
+        self.tms = TMSTileSource(TileGrid(), client)
     
     def test_id(self):
         eq_(self.tms.id(), 'http://%s:%s/tms' % TEST_SERVER_ADDRESS)
@@ -377,7 +413,7 @@ class TestTMSTileSource(object):
                 try:
                     self.tms.create_tile(_Tile((1, 0, 1)), lambda x: _Tile(x))
                 except TileSourceError, e:
-                    assert 'image' in e.message
+                    assert 'image' in e.args[0]
                 else:
                     assert False, 'expected TileSourceError'
 
@@ -441,7 +477,7 @@ class TestWMSTileSource(object):
             try:
                 self.wms.create_tile(_Tile((1, 0, 1)), lambda x: _Tile(x))
             except TileSourceError, e:
-                assert 'image' in e.message
+                assert 'image' in e.args[0]
             else:
                 assert False, 'expected TileSourceError'
     def test_get_tile_non_image_result(self):
@@ -453,8 +489,8 @@ class TestWMSTileSource(object):
             try:
                 self.wms.create_tile(_Tile((1, 0, 1)), lambda x: _Tile(x))
             except TileSourceError, e:
-                print e.message
-                assert 'cannot identify' in e.message
+                print e.args[0]
+                assert 'cannot identify' in e.args[0]
             else:
                 assert False, 'expected TileSourceError'
     
