@@ -20,7 +20,7 @@ Service requests (parsing, handling, etc).
 from mapproxy.wms import exceptions
 from mapproxy.core.config import base_config
 from mapproxy.core.exceptions import RequestError
-from mapproxy.core.srs import SRS
+from mapproxy.core.srs import SRS, make_lin_transf
 from mapproxy.core.request import RequestParams, BaseRequest, split_mime_type
 
 import logging
@@ -75,9 +75,18 @@ class WMSMapRequestParams(RequestParams):
     del _get_size
     del _set_size
     
-    @property
-    def srs(self):
+    def _get_srs(self):
         return self.params.get('srs', None)
+    def _set_srs(self, srs):
+        if hasattr(srs, 'srs_code'):
+            self.params['srs'] = srs.srs_code
+        else:
+            self.params['srs'] = srs
+    
+    srs = property(_get_srs, _set_srs)
+    del _get_srs
+    del _set_srs
+    
     
     @property
     def transparent(self):
@@ -119,10 +128,12 @@ class WMSRequest(BaseRequest):
     request_handler_name = None
     fixed_params = {}
     expected_param = []
+    non_strict_params = set()
     #pylint: disable-msg=E1102
     xml_exception_handler = None
     
-    def __init__(self, param=None, url='', validate=False):
+    def __init__(self, param=None, url='', validate=False, non_strict=False):
+        self.non_strict = non_strict
         BaseRequest.__init__(self, param=param, url=url, validate=validate)
         self.adapt_to_111()
     
@@ -158,12 +169,15 @@ class WMSMapRequest(WMSRequest):
     #pylint: disable-msg=E1102
     xml_exception_handler = None
 
-    def __init__(self, param=None, url='', validate=False):
-        WMSRequest.__init__(self, param=param, url=url, validate=validate)
+    def __init__(self, param=None, url='', validate=False, non_strict=False):
+        WMSRequest.__init__(self, param=param, url=url, validate=validate,
+                            non_strict=non_strict)
     
     def validate(self):
         missing_param = []
         for param in self.expected_param:
+            if self.non_strict and param in self.non_strict_params:
+                continue
             if param not in self.params:
                 missing_param.append(param)
         
@@ -301,7 +315,7 @@ class WMSFeatureInfoRequestParams(WMSMapRequestParams):
         return sum((layers.split(',') for layers in self.params.get_all('query_layers')), [])
 
     def _get_pos(self):
-        """x, y query image coordinates"""
+        """x, y query image coordinates (in pixel)"""
         return int(self['x']), int(self['y'])
     def _set_pos(self, value):
         self['x'] = str(int(round(value[0])))
@@ -309,11 +323,25 @@ class WMSFeatureInfoRequestParams(WMSMapRequestParams):
     pos = property(_get_pos, _set_pos)
     del _get_pos
     del _set_pos
+    
+    @property
+    def pos_coords(self):
+        """x, y query coordinates (in request SRS)"""
+        width, height = self.size
+        bbox = self.bbox
+        return make_lin_transf((0, 0, width, height), bbox)(self.pos)
 
 class WMS130FeatureInfoRequestParams(WMSFeatureInfoRequestParams):
     switch_bbox = _switch_bbox
 
-class WMS111FeatureInfoRequest(WMSMapRequest):
+class WMSFeatureInfoRequest(WMSMapRequest):
+    non_strict_params = set(['format', 'styles'])
+    
+    def validate_format(self):
+        if self.non_strict: return 
+        WMSMapRequest.validate_format(self)
+
+class WMS111FeatureInfoRequest(WMSFeatureInfoRequest):
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exceptions.WMS111ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -321,7 +349,7 @@ class WMS111FeatureInfoRequest(WMSMapRequest):
     fixed_params['request'] = 'GetFeatureInfo'
     expected_param = WMSMapRequest.expected_param[:] + ['query_layers', 'x', 'y']
     
-class WMS100FeatureInfoRequest(WMSMapRequest):
+class WMS100FeatureInfoRequest(WMSFeatureInfoRequest):
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exceptions.WMS100ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -334,25 +362,47 @@ class WMS100FeatureInfoRequest(WMSMapRequest):
     
     def adapt_params_to_version(self):
         params = WMSMapRequest.adapt_params_to_version(self)
-        del self.params['version']
+        del params['version']
         return params
 
-
-class WMS130FeatureInfoRequest(WMSMapRequest):
+class WMS130FeatureInfoRequest(WMS130MapRequest):
     request_params = WMS130FeatureInfoRequestParams
     xml_exception_handler = exceptions.WMS130ExceptionHandler
     request_handler_name = 'featureinfo'
     fixed_params = WMS130MapRequest.fixed_params.copy()
     fixed_params['request'] = 'GetFeatureInfo'
-    expected_param = WMS130MapRequest.expected_param[:] + ['query_layers', 'x', 'y']
+    expected_param = WMS130MapRequest.expected_param[:] + ['query_layers', 'i', 'j']
+    non_strict_params = set(['format', 'styles'])
+
+    def adapt_to_111(self):
+        WMS130MapRequest.adapt_to_111(self)
+        # only set x,y when present,
+        # avoids empty values for request templates
+        if 'i' in self.params:
+            self.params['x'] = self.params['i']
+        if 'j' in self.params:
+            self.params['y'] = self.params['j']
+        del self.params['i']
+        del self.params['j']
     
+    def adapt_params_to_version(self):
+        params = WMS130MapRequest.adapt_params_to_version(self)
+        params['i'] = self.params['x']
+        params['j'] = self.params['y']
+        del params['x']
+        del params['y']
+        return params
+    
+    def validate_format(self):
+        if self.non_strict: return
+        WMS130MapRequest.validate_format(self)
 
 class WMSCapabilitiesRequest(WMSRequest):
     request_handler_name = 'capabilities'
     exception_handler = None
     mime_type = 'text/xml'
     fixed_params = {}
-    def __init__(self, param=None, url='', validate=False):
+    def __init__(self, param=None, url='', validate=False, non_strict=False):
         WMSRequest.__init__(self, param=param, url=url, validate=validate)
     
     def adapt_to_111(self):
@@ -470,6 +520,7 @@ def negotiate_version(version):
             return next_highest_version
 
 def wms_request(req, validate=True):
+    non_strict = base_config().wms.non_strict
     version = _parse_version(req)
     req_type = _parse_request_type(req)
     
@@ -483,4 +534,5 @@ def wms_request(req, validate=True):
         dummy_req = version_requests['map'](param=req.args, url=req.base_url,
                                             validate=False)
         raise RequestError("unknown WMS request type '%s'" % req_type, request=dummy_req)
-    return req_class(param=req.args, url=req.base_url, validate=True)
\ No newline at end of file
+    return req_class(param=req.args, url=req.base_url, validate=True,
+                     non_strict=non_strict)
\ No newline at end of file
